"""
This module contains different camera specifications.

Especially perspective and orthographic cameras.
"""

from .camera import Camera
from .perspectivecamera import PerspectiveCamera
from .orthographiccamera import OrthographicCamera
