"""
This is the geometry module, containing maily shapes.

Also the vertex specification is found here.
"""

from .vertex import Vertex
from .shape import Shape
from .triangle import Triangle
from .sphere import Sphere
